%% Data analysis script for the SLANTS raw data (RIS ID: 10094).
% By Kristoffer Aalstad, September 2015.

%% Read in the data.
close all; clear all; clc;
ncfile='bay_Mar_20_25_2015.nc';
T=ncread(ncfile,'Temperature');    % Temperature [deg C].
t=ncread(ncfile,'Timestamp');      % Timestamp [datevec format].
t=datenum(t);                      % -> datenum format.
lat=ncread(ncfile,'Latitude');     % Latitude [deg N].
lon=ncread(ncfile,'Longitude');    % Longitude [deg E].
latlon=[lat lon];     % Combine lat & lon into single array.

%% Define the geometry of the objective analysis problem. 

% Create the analysis grid vectors:
lon0=11.82; lonx=11.84; 		% Minimum and maximum longitude.
lat0=78.922; latx=78.926; 		% Minimum and maximum latitude.
glon=(lon0:0.0005:lonx)';		% Analysis grid longitude vector.
glat=(lat0:0.0001:latx)';		% Analysis grid latitude vector.
[glon,glat]=meshgrid(glon,glat);
I=size(glon,1);   			      			% Number of latitudes in the grid.
J=size(glat,1);   			      			% Number of longitudes in the grid.
K=size(latlon,1);			      			% Number of observations.
N=size(t,1);	  						    % Number of points in time.


% Convert analysis grid vectors from latitudes and longitudes to distances in meters, using that
% the extent of the domain is small enough to ensure that the grid spacing (in meters) does not change considerably for 
% different latitudes.
dx=haversine([glat(1,1) glon(1,1)],[glat(1,2) glon(1,2)]); 	% Zonal grid spacing (in meters).
dy=haversine([glat(1,1) glon(1,1)],[glat(2,1) glon(2,1)]); 	% Meridional grid spacing (in meters).
X=((1:I)-1).*dx; % Zonal grid vector (distance in meters from the eastern most point).
Y=((1:J)-1).*dy; % Meridional grid vector (distance in meters from southern most point).

% Position of the observations on the grid (in meters).
XO=zeros(K,1); YO=XO;
for k=1:K
	YO(k,1)=haversine([glat(1,1) glon(1,1)],[latlon(k,1) glon(1,1)]);
	XO(k,1)=haversine([glat(1,1) glon(1,1)],[glat(1,1) latlon(k,2)]);
end

% Calculate the weights for Cressman's inverse distance weighting method; these only
% depend on the distance between the observations and the analysis point.
r=zeros(I,J,K); % Array to hold the distance between the analysis point at X(i) Y(j) and the observation at XO(K) YO(K).
w=r;            % Array to hold the corresponding weight of an observation at a given analysis point.
ro=nan(K,K);	% Array to hold the distance between the observations themselves.

% Calculate this observational separation vector:
for i=1:K
	for j=1:i
		ro(i,j)=haversine(latlon(i,:),latlon(j,:));
	end
end
ro=ro(:);
R=nanmedian(ro);% Set the radius of inluence as the median of the spacing between observations. 


% Calculate the Cressman weights.
for k=1:K % Observation loop.
	for j=1:J % Meridional loop.
		for i=1:I % Zonal loop.
			r(i,j,k)=haversine([glat(i,j) glon(i,j)],latlon(k,:));	% Calculate the separation distances.
			if r(i,j,k)<R % Only give the observations that are within a radius of influence
				      % from the analysis point weight.
				w(i,j,k)=(R^2-r(i,j,k)^2)/(R^2+r(i,j,k)^2); 
			else
			 	r(i,j,k)=0; % Already zero but set explicity here for clarity.
			end
			if k==K
				w(i,j,:)=w(i,j,:).*(1/sum(w(i,j,:)));	% Normalize the weights ensuring that these sum to 1.
			end
		end
	end
end


%% Perform the objective analysis and estimate the area averaged horizontal temperature gradients.
Tcress=zeros(I,J,N);   % Array to hold the analysis.
dTdx=zeros(I-1,J-1,N); % Array to hold zonal temperature gradient.
dTdy=zeros(I-1,J-1,N); % Array to hold meridional temperature gradient.
gradm=zeros(N,1);      % Array to hold the magnitude of the area averaged horizontal temperature gradient.
for n=1:N	% Temporal loop.
	for j=1:J % Meridional loop.
		for i=1:I % Zonal loop.
			Tcress(i,j,n)=sum(squeeze(w(i,j,:)).*(squeeze(T(n,:))')); 
			if j>1 && i>1
				% Perform linear interpoaltion between analysis points:
				Tl=0.5*(Tcress(i-1,j,n)+Tcress(i-1,j-1,n));	%   x	Tu  x		where x=points on the grid.  
				Tr=0.5*(Tcress(i,j,n)+Tcress(i,j-1,n));		%   Tl	o  Tr		and o=point where the
				Tu=0.5*(Tcress(i-1,j,n)+Tcress(i,j,n));		%   x   Tb  x	        gradient is estimated.
				Tb=0.5*(Tcress(i-1,j-1,n)+Tcress(i,j-1,n));

				% Approximate the horizontal temperature gradients as centered (second order) finite differences of the
				% interpolated temperatures.
				dTdx(i,j,n)=(Tr-Tl)/dx;
				dTdy(i,j,n)=(Tu-Tb)/dy;
			end
		end
    end
    zmdTdx=squeeze(mean(dTdx(:,:,n),1)); 	    % Area averaged zonal temperature gradient.
    zmdTdy=squeeze(mean(dTdy(:,:,n),1));	    % Area averaged meridional temperature gradient.
    gradm(n,1)=sqrt(mean(zmdTdx)^2+mean(zmdTdy)^2); % Calculate the magnitude of the area averaged horizontal temperature gradient.
end



%% Visualization;


sgradm=sort(gradm);
nis=find(abs(gradm-mean(gradm)-1*std(gradm))==min(abs(gradm-mean(gradm)-1*std(gradm))));
figure('name','Example Field','units','normalized','outerposition',[0.2 0.2 0.8 0.8])
hold on;
imagesc(X,Y,Tcress(:,:,nis));
plot(XO,YO,'.','Color',[1 1 1],'MarkerSize',35);
xlabel('Easting [m]','Interpreter','Latex','FontSize',18);
ylabel('Northing [m]','Interpreter','Latex','FontSize',18);
title(datestr(t(nis,1)),'Interpreter','Latex','FontSize',18);
xlim([min(X) max(X)]);
ylim([min(Y) max(Y)]);
colormap jet;
axis xy;
c=colorbar;
xlabel(c,'Estimated Temperature [$^\circ$C]','Interpreter','Latex','FontSize',18);
axis('square');
pause(0.01);
grid on;
box on;
set(gca,'TickDir','out');
print('-dpdf','Cressman','-opengl','-r300');





FA=(2*1.2*1e3).*sgradm; % Scale the temp gradient to recover an estimate  
                        % for the horizontal advective heat flux density.
FAall=FA; FA=FA(FA<8);
figure('name','distribution','units','normalized','outerposition',[0.2 0.2 0.8 0.8])
nbins=8;
l1=histogram(FA,nbins,'Normalization','pdf','FaceColor',[0.7 0.7 0.7],...
    'EdgeColor',[0 0 0]);
hold on;
scatter(FA,zeros(size(FA)),400,'.','MarkerEdgeColor',[0.0 0.0 0.0]);
pd = fitdist(FA,'Weibull');
pdffit=pdf(pd,FA);
draws=wblrnd(pd.A,pd.B,size(FA));
[h,p]=kstest2(draws,FA);
hold on;
alpha(0.1);
l2=plot(FA,pdffit,'LineWidth',3,'Color',[0 0 0.8]);
grid on;
box on;
legstr={'Sample Distribution',sprintf('$W(A=%4.2f,k=%4.2f)$',pd.A,pd.B)};
leg=legend([l1;l2],legstr,'Interpreter','Latex','FontSize',14,'Location','NorthEast');
xlabel('Horizontal Advective Heat Flux Density, $F_{A}$   [Wm$^{-2}$]','Interpreter','Latex',...
	'FontSize',16);
xlim([0 7]);
ylabel('Probability Density','Interpreter','Latex','FontSize',18);
tis=sprintf('$\\mu_{F_{A}}=%4.2f,  \\sigma_{F_{A}}=%4.2f, S_{F_{A}}=%4.2f,  K_{F_{A}}=%4.2f$'...
    ,mean(FAall),std(FAall),skewness(FAall),kurtosis(FAall));
title(tis,'Interpreter','Latex','FontSize',16);
axis('square');
set(gca,'TickDir','out');
print('-dpdf','FA','-opengl','-r300');

